import io
import typing as t
from datetime import datetime
from zipfile import ZipFile

from mealieapi.auth import Token
from mealieapi.const import YEAR_MONTH_DAY, YEAR_MONTH_DAY_HOUR_MINUTE_SECOND
from mealieapi.meals import Ingredient, Meal, MealPlan, MealPlanDay, ShoppingList
from mealieapi.misc import DebugInfo, DebugStatistics, DebugVersion, File
from mealieapi.raw import RawClient
from mealieapi.recipes import (
    Recipe,
    RecipeAsset,
    RecipeCategory,
    RecipeComment,
    RecipeImage,
    RecipeNutrition,
    RecipeTag,
)
from mealieapi.users import Group, User, UserSignup


class MealieClient(RawClient):
    # User API Keys
    def process_token_json(self, data: t.Dict[str, t.Any]) -> Token:
        return Token(self, **data)

    async def create_api_key(self, name: str) -> Token:
        data = await self.request(
            "users/api-tokens", method="POST", json=dict(name=name)
        )
        return self.process_token_json(data)

    async def delete_api_key(self, id: int) -> None:
        await self.request(f"users/api-tokens/{id}", method="DELETE")

    # User Signups
    async def signup_with_token(self, token: str, user: User) -> User:
        data = await self.request(
            f"users/sign-ups/{token}", method="POST", json=user.json(), use_auth=False
        )
        return self.process_user_json(data)

    async def delete_signup_token(self, token: t.Optional[str]) -> None:
        if token is None:
            await self.request(f"users/sign-ups/{token}", method="DELETE")
        else:
            raise ValueError("Token string cannot be NoneType must be string")

    async def get_open_signups(self) -> t.List[UserSignup]:
        data = await self.request("users/sign-ups")
        return [UserSignup(self, **signup) for signup in data]

    async def create_signup_token(self, name: str, admin: bool = False) -> UserSignup:
        data = await self.request(
            "users/sign-ups", method="POST", json=dict(name=name, admin=admin)
        )
        return UserSignup(self, **data)

    # Users
    def process_user_json(self, data: t.Dict[str, t.Any]) -> User:
        if data.get("favorite_recipes"):
            data["favorite_recipes"] = [
                self.process_recipe_json(recipe) for recipe in data["favorite_recipes"]
            ]
        if data.get("tokens"):
            data["tokens"] = [
                self.process_token_json(token_data) for token_data in data["tokens"]
            ]
        return User(self, **data)

    async def get_user_image(self, user_id: int) -> bytes:
        return await self.request(f"users/{user_id}/image", use_auth=False)

    async def update_user_image(self, user_id: int, file: io.BytesIO) -> bytes:
        return await self.request(
            f"users/{user_id}/image", data=dict(profile_image=file)  # type: ignore[arg-type]
        )

    async def get_user(self, user_id: int) -> User:
        data = await self.request(f"users/{user_id}")
        return self.process_user_json(data)

    async def update_user(self, user_id: int, user: User) -> User:
        data = await self.request(f"users/{user_id}", method="PUT", json=user.json())
        return self.process_user_json(data)

    async def delete_user(self, user_id: int) -> None:
        await self.request(f"users/{user_id}", method="DELETE")

    async def reset_password(self, user_id: int) -> None:
        await self.request(f"users/{user_id}/reset-password", method="PUT")

    async def update_password(
        self, user_id: int, current_password: str, new_password: str
    ) -> None:
        await self.request(
            f"users/{user_id}/password",
            method="PUT",
            json=dict(current_password=current_password, new_password=new_password),
        )

    async def get_favorites(self, user_id: int) -> t.Optional[t.List[Recipe]]:
        data = await self.request(f"users/{user_id}/favorites")
        user = self.process_user_json(data)
        return user.favorite_recipes

    async def add_favorite(self, user_id: int, recipe_slug: str) -> None:
        await self.request(f"users/{user_id}/favorites/{recipe_slug}", method="POST")

    async def remove_favorite(self, user_id: int, recipe_slug: str):
        await self.request(f"users/{user_id}/{recipe_slug}", method="DELETE")

    async def get_all_users(self) -> t.List[User]:
        data = await self.request("users")
        return [self.process_user_json(user) for user in data]

    async def create_user(self, user: User) -> User:
        data = await self.request("users", method="POST", json=user.json())
        return self.process_user_json(data)

    # Groups
    def process_group_json(self, data: t.Dict[str, t.Any]) -> Group:
        data["users"] = [self.process_user_json(info) for info in data["users"]]
        return Group(self, **data)

    async def get_groups(self) -> t.List[Group]:
        data = await self.request("groups")
        return [self.process_group_json(group) for group in data]

    async def create_group(self, group: Group) -> Group:
        data = await self.request("groups", method="POST", json=group.json())
        return self.process_group_json(data)

    async def update_group(self, id: int, group: Group) -> Group:
        data = await self.request(f"groups/{id}", method="PUT", json=group.json())
        return self.process_group_json(data)

    async def delete_group(self, id: int) -> None:
        await self.request(f"groups/{id}", method="DELETE")

    # Current User
    async def get_current_user(self) -> User:
        data = await self.request("users/self")
        return self.process_user_json(data)

    async def get_current_group(self) -> Group:
        data = await self.request("groups/self")
        return self.process_group_json(data)

    # Query All Recipes
    async def get_recipes(self, start=0, limit=9999) -> t.List[Recipe]:
        data = await self.request(
            "recipes/summary", params={"start": start, "limit": limit}
        )
        return [Recipe(self, **data) for data in data]

    async def get_untagged_recipes(self) -> t.List[Recipe]:
        data = await self.request("recipes/summary/untagged")
        return [Recipe(self, **recipe) for recipe in data]

    async def get_uncategorized_recipes(self) -> t.List[Recipe]:
        data = await self.request("recipes/summary/uncategorized")
        return [Recipe(self, **recipe) for recipe in data]

    # Recipe Methods
    def process_comment_json(self, data: t.Dict[str, t.Any]) -> RecipeComment:
        data["date_added"] = datetime.strptime(
            data["date_added"], YEAR_MONTH_DAY_HOUR_MINUTE_SECOND
        )
        data["user"] = self.process_user_json(data["user"])
        return RecipeComment(self, **data)

    def process_nutrition_json(self, data: t.Dict[str, t.Any]) -> RecipeNutrition:
        return RecipeNutrition(**data)

    def process_recipe_json(self, data: t.Dict[str, t.Any]) -> Recipe:
        data["org_url"] = data.get("org_u_r_l")
        if "org_u_r_l" in data:
            del data["org_u_r_l"]
        if data["comments"]:
            data["comments"] = [
                self.process_comment_json(comment) for comment in data["comments"]
            ]
        if data["date_added"]:
            data["date_added"] = datetime.strptime(data["date_added"], YEAR_MONTH_DAY)
        if data["date_updated"]:
            data["date_updated"] = datetime.strptime(
                data["date_updated"], YEAR_MONTH_DAY_HOUR_MINUTE_SECOND
            )
        del data["slug"]
        return Recipe(self, **data)

    async def get_recipe(self, recipe_slug: str) -> Recipe:
        data = await self.request(f"recipes/{recipe_slug}")
        return self.process_recipe_json(data)

    async def delete_recipe(self, recipe_slug: str) -> Recipe:
        data = await self.request(f"recipes/{recipe_slug}", method="DELETE")
        return self.process_recipe_json(data)

    async def update_recipe(self, recipe: Recipe) -> Recipe:
        data = await self.request(
            f"recipes/{recipe.slug}", method="PUT", json=recipe.json()
        )
        return self.process_recipe_json(data)

    async def get_recipe_zip(self, recipe_slug: str) -> ZipFile:
        data = await self.request(f"recipes/{recipe_slug}/zip", use_auth=False)
        return ZipFile(io.BytesIO(data))

    async def create_recipe(self, recipe: Recipe) -> Recipe:
        slug = await self.request("recipes/create", json=recipe.json(), method="POST")
        return await self.get_recipe(slug)

    async def create_recipe_from_url(self, url: str) -> Recipe:
        slug = await self.request(
            "recipes/create-url", method="POST", json=dict(url=url)
        )
        return await self.get_recipe(slug)

    async def create_recipe_from_zip(self, file: io.BytesIO):
        # TODO: Look into zip file encoding to fix OSError: [Errno 22] Invalid Argument from Server logs
        slug = await self.request(
            "recipes/create-from-zip", method="POST", data={"archive": file}  # type: ignore[arg-type]
        )
        return await self.get_recipe(slug)

    # Recipe Images
    async def update_recipe_image(
        self, recipe_slug: str, file: io.BytesIO, extension: str
    ) -> RecipeImage:
        data = await self.request(
            f"recipes/{recipe_slug}/image",
            data={"image": file, "extension": extension},  # type: ignore[arg-type]
            method="PUT",
        )
        return RecipeImage(self, recipe_slug, **data)

    async def update_recipe_image_from_url(self, recipe_slug: str, url: str) -> None:
        await self.request(
            f"recipes/{recipe_slug}/image", method="POST", json=dict(url=url)
        )

    async def upload_recipe_asset(
        self, recipe_slug: str, name: str, icon: str, extension: str, file: io.BytesIO
    ) -> RecipeAsset:
        data = await self.request(
            f"recipes/{recipe_slug}/assets",
            data=dict(name=name, icon=icon, extension=extension, file=file),  # type: ignore[arg-type]
            method="POST",
        )
        return RecipeAsset(self, recipe_slug, **data)

    # Recipe Tags
    def process_tag_json(self, data: t.Dict[str, t.Any]) -> RecipeTag:
        if data.get("recipes"):
            data["recipes"] = [
                self.process_recipe_json(recipe) for recipe in data["recipes"]
            ]
        if data.get("slug"):
            del data["slug"]
        return RecipeTag(self, **data)

    async def get_tags(self) -> t.List[RecipeTag]:
        tags = await self.request("tags", use_auth=False)
        return [self.process_tag_json(data) for data in tags]

    async def create_tag(self, name: str) -> RecipeTag:
        data = await self.request("tags", method="POST", json=dict(name=name))
        return self.process_tag_json(data)

    async def get_empty_tags(self) -> t.List[RecipeTag]:
        tags = await self.request("tags/empty", use_auth=False)
        return [self.process_tag_json(data) for data in tags]

    async def get_tag_recipes(self, tag_slug: str) -> t.List[Recipe]:
        data = await self.request(f"tags/{tag_slug}")
        return [self.process_recipe_json(recipe) for recipe in data["recipes"]]

    async def update_tag(self, tag_slug: str, new_name: str) -> RecipeTag:
        data = await self.request(
            f"tags/{tag_slug}", method="PUT", json=dict(name=new_name)
        )
        return self.process_tag_json(data)

    async def delete_tag(self, tag_slug: str) -> None:
        await self.request(f"tags/{tag_slug}", method="DELETE")

    # Recipe Categories
    def process_category_json(self, data: t.Dict[str, t.Any]) -> RecipeCategory:
        if data.get("recipes"):
            data["recipes"] = [
                self.process_recipe_json(recipe) for recipe in data["recipes"]
            ]
        if data.get("slug"):
            del data["slug"]
        return RecipeCategory(self, **data)

    async def get_categories(self) -> t.List[RecipeCategory]:
        categories = await self.request("categories", use_auth=False)
        return [self.process_category_json(data) for data in categories]

    async def create_category(self, name: str) -> RecipeCategory:
        data = await self.request("categories", method="POST", json=dict(name=name))
        return self.process_category_json(data)

    async def get_empty_categories(self) -> t.List[RecipeCategory]:
        categories = await self.request("categories/empty", use_auth=False)
        return [self.process_category_json(data) for data in categories]

    async def get_category_recipes(self, category_slug: str) -> t.List[Recipe]:
        data = await self.request(f"categories/{category_slug}")
        return [self.process_recipe_json(recipe) for recipe in data["recipes"]]

    async def update_category(
        self, category_slug: str, new_name: str
    ) -> RecipeCategory:
        data = await self.request(
            f"categories/{category_slug}", method="PUT", json=dict(name=new_name)
        )
        return self.process_category_json(data)  # type: ignore[arg-type]

    async def delete_category(self, category_slug: str) -> None:
        await self.request(f"categories/{category_slug}", method="DELETE")

    # Recipe Comments
    async def create_recipe_comment(
        self, recipe_slug: str, comment: RecipeComment
    ) -> RecipeComment:
        data = await self.request(
            f"recipes/{recipe_slug}/comments", method="POST", data=comment.json()  # type: ignore[arg-type]
        )
        return self.process_comment_json(data)  # type: ignore[arg-type]

    async def update_recipe_comment(
        self, recipe_slug: str, comment_id: int, comment: RecipeComment
    ) -> RecipeComment:
        data = await self.request(
            f"recipes/{recipe_slug}/comments/{comment_id}",
            method="PUT",
            data=comment.json(),  # type: ignore[arg-type]
        )
        return self.process_comment_json(data)  # type: ignore[arg-type]

    async def delete_recipe_comment(self, recipe_slug: str, comment_id: int):
        await self.request(
            f"recipes/{recipe_slug}/comments/{comment_id}", method="DELETE"
        )

    # Shopping List
    def process_shopping_list_json(self, data: t.Dict[str, t.Any]) -> ShoppingList:
        data["items"] = [Ingredient(self, **item) for item in data["items"]]
        return ShoppingList(self, **data)

    async def create_shopping_list(self, shopping_list: ShoppingList) -> ShoppingList:
        data = await self.request(
            "shopping-lists", method="POST", json=shopping_list.json()
        )
        return self.process_shopping_list_json(data)  # type: ignore[arg-type]

    async def get_shopping_list(self, id: int) -> ShoppingList:
        data = await self.request(f"shoppings-list/{id}")
        return self.process_shopping_list_json(data)  # type: ignore[arg-type]

    async def update_shopping_list(
        self, id: int, shopping_list: ShoppingList
    ) -> ShoppingList:
        data = await self.request(
            f"shopping-lists/{id}", method="PUT", json=shopping_list.json()
        )
        return self.process_shopping_list_json(data)  # type: ignore[arg-type]

    async def delete_shopping_list(self, id: int) -> None:
        await self.request(f"shopping-lists/{id}", method="DELETE")

    # Meal Plans
    def process_meal_json(self, data: t.Dict[str, t.Any]) -> Meal:
        return Meal(self, **data)

    def process_mealplanday_json(self, data: t.Dict[str, t.Any]) -> MealPlanDay:
        data["date"] = datetime.strptime(data["date"], YEAR_MONTH_DAY)
        data["meals"] = [self.process_meal_json(meal) for meal in data["meals"]]
        return MealPlanDay(self, **data)

    def process_mealplan_json(self, data: t.Dict[str, t.Any]) -> MealPlan:
        data["end_date"] = datetime.strptime(data["end_date"], YEAR_MONTH_DAY)
        data["start_date"] = datetime.strptime(data["end_date"], YEAR_MONTH_DAY)
        data["plan_days"] = [
            self.process_mealplanday_json(day) for day in data["plan_days"]
        ]
        return MealPlan(self, **data)

    async def get_mealplans_all(self) -> t.List[MealPlan]:
        data = await self.request("meal-plans/all")
        return [self.process_mealplan_json(mealplan) for mealplan in data]  # type: ignore[arg-type]

    async def get_mealplan_this_week(self) -> MealPlan:
        data = await self.request("meal-plans/this-week")
        return self.process_mealplan_json(data)  # type: ignore[arg-type]

    async def get_todays_meal(self) -> Recipe:
        data = await self.request("meal-plans/today")
        return await self.get_recipe(data.decode())  # type: ignore[arg-type]

    async def get_mealplan(self, id: int) -> MealPlan:
        data = await self.request(f"meal-plans/{id}")
        return self.process_mealplan_json(data)  # type: ignore[arg-type]

    async def update_mealplan(self, id: int, mealplan: MealPlan) -> MealPlan:
        data = await self.request(
            f"meal-plans/{id}", method="PUT", json=mealplan.json()
        )
        return self.process_mealplan_json(data)  # type: ignore[arg-type]

    async def create_mealplan(self, mealplan: MealPlan) -> MealPlan:
        data = await self.request("meal-plans", method="POST", json=mealplan.json())
        return self.process_mealplan_json(data)  # type: ignore[arg-type]

    async def delete_mealplan(self, id: int) -> None:
        await self.request(f"meal-plans/{id}", method="DELETE")

    async def get_todays_meal_image(self) -> bytes:
        return await self.request("meal-plans/today/image")  # type: ignore[arg-type]

    async def get_mealplan_shopping_list(self, id: int) -> ShoppingList:
        data = await self.request(f"meal-plans/{id}")
        return self.process_shopping_list_json(data)  # type: ignore[arg-type]

    # Site Media
    async def get_asset(self, recipe_slug: str, file_name: str) -> bytes:
        return await self.request(f"media/recipes/{recipe_slug}/assets/{file_name}")  # type: ignore[arg-type]

    async def get_image(self, recipe_slug: str, type="original") -> bytes:
        """
        Gets the image for the recipe.
        Valid types are :code:`original`, :code:`min-original`, and :code:`tiny-original`
        """
        return await self.request(f"media/recipes/{recipe_slug}/images/{type}.webp")  # type: ignore[arg-type]

    # Debug
    async def get_log_file(self) -> File:
        data = await self.request("debug/log")
        return File(self, str(data.get("file_token")))  # type: ignore[arg-type]

    async def get_debug(self) -> DebugInfo:
        data = await self.request("debug")
        return DebugInfo(**data)  # type: ignore[arg-type]

    async def get_debug_version(self) -> DebugVersion:
        data = await self.request("debug/version", use_auth=False)
        return DebugVersion(**data)  # type: ignore[arg-type]

    async def get_debug_statistics(self) -> DebugStatistics:
        data = await self.request("debug/statistics")
        return DebugStatistics(**data)  # type: ignore[arg-type]

    # Misc
    async def download_file(self, file_token: str) -> bytes:
        content = await self.request(
            "utils/download", params=dict(token=file_token), use_auth=False
        )
        return content  # type: ignore[arg-type]



import typing as t
from dataclasses import dataclass, field

import aiohttp

from mealieapi.mixins import JsonModel

if t.TYPE_CHECKING:
    from mealieapi.client import MealieClient
    from mealieapi.raw import RawClient


@dataclass()
class Token(JsonModel):
    _client: "MealieClient" = field(repr=False)
    name: str
    id: int
    token: t.Union[str, None] = None

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json({"name", "id"})

    async def delete(self) -> None:
        await self._client.delete_api_key(self.id)


@dataclass()
class Auth:
    _client: "RawClient" = field(repr=False)
    access_token: str
    token_type: t.Optional[str] = None

    async def refresh(self):
        resp = await self._client.request("auth/refresh")
        self.access_token = resp["access_token"]
        self.token_type = resp["token_type"]

    @property
    def header(self):
        return {aiohttp.hdrs.AUTHORIZATION: f"Bearer {self.access_token}"}



import logging
import os
import typing as t

import aiohttp

from mealieapi.auth import Auth
from mealieapi.errors import (
    BadRequestError,
    InternalServerError,
    MealieError,
    ParameterMissingError,
    UnauthenticatedError,
)
from mealieapi.misc import camel_to_snake_case


class _RawClient:
    auth: t.Optional[Auth] = None
    response_processors: t.Dict[str, t.Callable] = {}

    def __init__(self, url: str) -> None:
        self.url = url

    def endpoint(self, path: str) -> str:
        return os.path.join(self.url, "api", path)

    def _headers(self) -> dict:
        return {
            aiohttp.hdrs.ACCEPT: "application/json",
            aiohttp.hdrs.USER_AGENT: "MealieAPI-Python 0.0.0",
        }

    async def request(
        self,
        path: str,
        method: str = "GET",
        data: t.Union[str, None] = None,
        json: t.Union[dict, None] = None,
        params: t.Union[dict, None] = None,
        use_auth: bool = True,
        **kwargs,
    ) -> t.Any:
        headers = self._headers()
        if use_auth is False and self.auth is not None:
            del headers[aiohttp.hdrs.AUTHORIZATION]
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.request(
                method=method,
                url=self.endpoint(path),
                data=data,
                json=json,
                params=params,
                **kwargs,
            ) as response:
                return await self.process_response(response)

    @staticmethod
    def response_processor(mimetype: str) -> t.Callable:
        def register_processor(processor: t.Callable):
            _RawClient.response_processors[mimetype] = processor
            return processor

        return register_processor

    async def process_response(self, response: aiohttp.ClientResponse) -> t.Any:
        logging.debug(f"Status: {response.status}")
        logging.debug(f"URL: {response.url}")
        logging.debug(f"Method: {response.method}")
        logging.debug(f"Content: {await response.read()!r}"[:100] + "...")
        logging.debug(response.request_info)
        if 200 <= response.status < 300:

            async def default_handler(response: aiohttp.ClientResponse) -> bytes:
                return await response.read()

            content_type = response.headers.get(aiohttp.hdrs.CONTENT_TYPE)
            if content_type is not None:
                processor = self.response_processors.get(content_type, default_handler)
            else:
                raise MealieError("Mealie did not return a content-type header.")
            return await processor(response)
        elif 400 <= response.status < 500:
            await self.handle_error_json(await response.json())
        else:
            raise InternalServerError("Mealie had a problem with your request.")

    async def handle_error_json(self, data: dict):
        if detail := data.get("detail", "Bad Request") == "Not authenticated":
            raise UnauthenticatedError("Not authenticated with Mealie")
        elif detail == "Bad Request":
            raise BadRequestError("Error with your request.")
        elif detail == "Internal Server Error":
            raise InternalServerError()
        elif isinstance(detail, list):
            for error in detail:
                if error["type"] == "value_error.missing":
                    params = error["loc"]
                    msg = error["msg"]
                    raise ParameterMissingError(
                        f"Missing the parameters {params!r}, {msg}"
                    )


@_RawClient.response_processor("application/json")
async def process_json(response: aiohttp.ClientResponse) -> t.Union[dict, str]:
    data = await response.json()
    if isinstance(data, dict) or isinstance(data, list):
        data = camel_to_snake_case(data)
    return data


@_RawClient.response_processor("application/octet-stream")
async def process_stream(response: aiohttp.ClientResponse) -> bytes:
    return await response.read()


class RawClient(_RawClient):
    def __init__(self, url) -> None:
        super().__init__(url)

    # Authorization
    def _headers(self) -> dict:
        """Updates the Raw Client headers with the Authorization header."""
        headers = super()._headers()
        if self.auth is not None:
            headers.update(self.auth.header)
        return headers

    async def _get_token(self, username: str, password: str) -> Auth:
        """Exchanges the login credentials of a user for a temporary API token."""
        data = await self.request(
            "auth/token",
            method="POST",
            data={"username": username, "password": password},  # type: ignore[arg-type]
            use_auth=False,
        )
        return Auth(self, **data)

    async def login(self, username: str, password: str) -> None:
        """Makes the Client authorize with the login credentials of a user."""
        self.auth = await self._get_token(username, password)

    def authorize(self, token: str) -> None:
        """Makes the Client authorize with an API token."""
        self.auth = Auth(self, token)



import re
import typing as t
from dataclasses import dataclass

if t.TYPE_CHECKING:
    from mealieapi.client import MealieClient


def name_to_slug(name: str) -> str:
    letters = filter(lambda char: char in "qwertyuiopasdfghjklzxcvbnm ", name.lower())
    return "".join(letters).replace(" ", "-")


def camel_to_snake_case(obj):
    if isinstance(obj, dict):
        for key, value in list(obj.items()):
            new_key = camel_to_snake_case(key)
            del obj[key]
            obj[new_key] = value
            if isinstance(value, dict):
                obj[new_key] = camel_to_snake_case(value)
        return obj
    elif isinstance(obj, list):
        return [camel_to_snake_case(item) for item in obj]
    elif isinstance(obj, str):
        return re.sub(r"(?<!^)(?=[A-Z])", "_", obj).lower()


@dataclass()
class File:
    _client: "MealieClient"
    file_token: str

    async def download(self) -> bytes:
        return await self._client.download_file(self.file_token)


@dataclass()
class DebugInfo:
    production: t.Optional[bool] = None
    version: t.Optional[str] = None
    demo_status: t.Optional[bool] = None
    api_port: t.Optional[int] = None
    api_docs: t.Optional[bool] = None
    db_type: t.Optional[str] = None
    db_url: t.Optional[str] = None
    default_group: t.Optional[str] = None


@dataclass()
class DebugStatistics:
    total_recipes: t.Optional[int] = None
    total_users: t.Optional[int] = None
    total_groups: t.Optional[int] = None
    uncategorized_recipes: t.Optional[int] = None
    untagged_recipes: t.Optional[int] = None


@dataclass()
class DebugVersion:
    production: t.Optional[bool] = None
    version: t.Optional[str] = None
    demo_status: t.Optional[bool] = None



class MealieError(Exception):
    pass


class UnauthenticatedError(MealieError):
    pass


class BadRequestError(MealieError):
    pass


class ParameterMissingError(MealieError):
    pass


class InternalServerError(MealieError):
    pass



import typing as t


class JsonModel:
    def json(self, attrs: t.Set[str]) -> t.Dict[str, t.Any]:
        data = {}
        for attr in attrs:
            value = getattr(self, attr)
            if value:
                if isinstance(value, JsonModel):
                    data[attr] = value.json()  # type: ignore[call-arg]
                elif isinstance(value, list):
                    data[attr] = [  # type: ignore[assignment]
                        element if not isinstance(value, JsonModel) else element.json()
                        for element in value
                    ]
                else:
                    data[attr] = value
        return data



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mealieapi']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.8.1,<4.0.0']

setup_kwargs = {
    'name': 'mealieapi',
    'version': '0.0.0',
    'description': 'Control your Mealie instance with python!',
    'long_description': '![mealie-image](https://hay-kot.github.io/mealie/assets/img/home_screenshot.png)\n\n# Mealie API\nIf you are running a self-hosted [Mealie](https://hay-kot.github.io/mealie/) server you can use this library to authenticate yourself with and intereact with it!\nCreate mealplans, import recipes, remove users, modify user groups, upload recipe images.\nAll with MealieAPI.\n\n## Installation\n\n\n## Usage\n\n\n### Authentication\nTo start you need your Mealie server url, and your login credentials or an API key (which you can create at `https://[YOUR_MEALIE_SERVER]/admin/profile`).\nMealieAPI uses the `async`/`await` syntax so you must run it inside an async function or event loop like so (if you are not familiar with async applications already.)\n\n\n```py\nimport asyncio\nfrom mealieapi import MealieClient\n\n\nclient = MealieClient("<YOUR_MEALIE_SERVER_ADDRESS>")\n```\nThis next part depends on whether you have an API key, or your login credentials.\n\nIf you want to use your username and password you must use `await client.login("<USERNAME_OR_EMAIL>", "<PASSWORD>")` or if you are using an API key you need to use `client.authorize("<API_KEY>")` (Note: without the await).\n\n```\nasync def main():\n    await client.login("<USERNAME_OR_EMAIL>", "<PASSWORD>")\n    # OR\n    client.authorize("<API_KEY>")\n\nloop = asyncio.get_event_loop()\nloop.run_until_complete(main())\n```\n\n\n',
    'author': 'GrandMoff100',
    'author_email': 'nlarsen23.student@gmail.com',
    'maintainer': 'GrandMoff100',
    'maintainer_email': 'nlarsen23.student@gmail.com',
    'url': 'https://github.com/GrandMoff100/MealieAPI',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



import typing as t
from dataclasses import dataclass, field
from datetime import datetime

from mealieapi.const import YEAR_MONTH_DAY
from mealieapi.misc import name_to_slug
from mealieapi.mixins import JsonModel

if t.TYPE_CHECKING:
    from mealieapi.client import MealieClient


@dataclass()
class Meal(JsonModel):
    _client: "MealieClient" = field(repr=False)
    name: str
    description: str

    @property
    def slug(self):
        return name_to_slug(self.name)

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json({"name", "slug", "description"})

    async def get_recipe(self):
        pass


@dataclass()
class MealPlanDay(JsonModel):
    _client: "MealieClient" = field(repr=False)
    date: datetime
    meals: t.List[Meal]

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        data = super().json({"date", "meals"})
        data["date"] = data["date"].strftime(YEAR_MONTH_DAY)
        return data


@dataclass()
class MealPlan(JsonModel):
    _client: "MealieClient" = field(repr=False)
    group: str
    end_date: datetime
    start_date: datetime
    meals: t.List[Meal]
    id: t.Union[int, None] = None
    shopping_list: t.Union[int, None] = None

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        data = super().json({"group", "end_date", "start_date", "meals"})
        data["end_date"] = data["end_date"].strftime(YEAR_MONTH_DAY)
        data["start_date"] = data["start_date"].strftime(YEAR_MONTH_DAY)
        return data


@dataclass()
class Ingredient(JsonModel):
    _client: "MealieClient" = field(repr=False)
    title: str
    text: str
    quantity: int
    checked: bool

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json({"title", "text", "quantity", "checked"})


@dataclass()
class ShoppingList(JsonModel):
    _client: "MealieClient" = field(repr=False)
    name: str
    group: str
    items: t.List[Ingredient]
    id: t.Union[int, None] = None

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json({"name", "groups", "items"})

    async def toggle_checked(self, index: int) -> "ShoppingList":
        self.items[index].checked = not self.items[index].checked
        return await self.update()

    def length(self):
        return len(self.items)

    async def create(self) -> "ShoppingList":
        return await self._client.create_shopping_list(self)

    async def update(self) -> "ShoppingList":
        if self.id is not None:
            return await self._client.update_shopping_list(self.id, self)
        else:
            raise ValueError("Missing required attribute id")

    async def delete(self) -> None:
        if self.id is not None:
            await self._client.delete_shopping_list(self.id)
        else:
            raise ValueError("Missing required attribute id")



import typing as t
from dataclasses import dataclass, field
from datetime import datetime
from zipfile import ZipFile

from mealieapi.const import YEAR_MONTH_DAY, YEAR_MONTH_DAY_HOUR_MINUTE_SECOND
from mealieapi.misc import name_to_slug
from mealieapi.mixins import JsonModel

if t.TYPE_CHECKING:
    from mealieapi.client import MealieClient
    from mealieapi.users import User


@dataclass()
class RecipeImage(JsonModel):
    _client: "MealieClient" = field(repr=False)
    recipe_slug: str
    image: int


@dataclass()
class RecipeAsset(JsonModel):
    _client: "MealieClient" = field(repr=False)
    recipe_slug: str
    file_name: str
    name: t.Optional[str] = None
    icon: t.Optional[str] = None

    async def content(self) -> bytes:
        return await self._client.get_asset(self.recipe_slug, self.file_name)


@dataclass()
class RecipeNutrition:
    calories: t.Optional[float] = None
    fatContent: t.Optional[float] = None
    proteinContent: t.Optional[float] = None
    carbohydrateContent: t.Optional[float] = None
    fiberContent: t.Optional[float] = None
    sodiumContent: t.Optional[float] = None
    sugarContent: t.Optional[float] = None


@dataclass()
class RecipeComment(JsonModel):
    _client: "MealieClient" = field(repr=False)
    recipe_slug: str
    text: str
    id: int
    uuid: str
    date_added: datetime
    user: "User"

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json(
            {
                "text",
            }
        )

    async def create(self) -> "RecipeComment":
        return await self._client.create_recipe_comment(self.recipe_slug, self)

    async def update(self, text: str) -> "RecipeComment":
        return await self._client.update_recipe_comment(self.recipe_slug, self.id, self)

    async def delete(self) -> None:
        await self._client.delete_recipe_comment(self.recipe_slug, self.id)


@dataclass(repr=False)
class Recipe(JsonModel):
    _client: "MealieClient" = field(repr=False)
    name: str
    description: t.Optional[str] = None
    image: t.Optional[str] = None
    recipe_yield: t.Optional[str] = None
    recipe_ingredient: t.Optional[t.List[str]] = None
    recipe_instructions: t.Optional[t.List[t.Dict[str, str]]] = None
    tags: t.Optional[t.List[str]] = None
    recipe_category: t.Optional[t.List[str]] = None
    notes: t.Optional[t.List[t.Dict[str, str]]] = None
    rating: t.Optional[int] = None
    extras: t.Optional[t.Dict[str, str]] = None
    id: t.Optional[int] = None
    settings: t.Optional[t.Dict[str, bool]] = None
    total_time: t.Optional[str] = None
    prep_time: t.Optional[str] = None
    perform_time: t.Optional[str] = None
    nutrition: t.Optional[RecipeNutrition] = None
    date_added: t.Optional[datetime] = None
    date_updated: t.Optional[datetime] = None
    org_url: t.Optional[str] = None
    tools: t.Optional[list] = None
    assets: t.Optional[t.List[RecipeAsset]] = None
    comments: t.Optional[t.List[RecipeComment]] = None

    @property
    def slug(self):
        return name_to_slug(self.name)

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        data = super().json(
            {
                "slug",
                "name",
                "description",
                "image",
                "recipe_yield",
                "recipe_ingredient",
                "recipe_instructions",
                "tags",
                "recipe_category",
                "notes",
                "org_url",
                "rating",
                "extras",
                "id",
                "settings",
                "total_time",
                "prep_time",
                "perform_time",
                "nutrition",
                "date_added",
                "date_updated",
                "tools",
                "assets",
                "comments",
            }
        )
        if data["date_added"]:
            data["date_added"] = data["date_added"].strftime(YEAR_MONTH_DAY)
        if data["date_updated"]:
            data["date_updated"] = data["date_updated"].strftime(
                YEAR_MONTH_DAY_HOUR_MINUTE_SECOND
            )
        return data

    async def create(self) -> "Recipe":
        return await self._client.create_recipe(self)

    async def delete(self) -> "Recipe":
        return await self._client.delete_recipe(self.slug)

    async def get_asset(self, file_name: str):
        return await self._client.get_asset(self.slug, file_name)

    async def get_image(self, type="original") -> t.Optional[bytes]:
        """
        Gets the image for the recipe.
        Valid types are :code:`original`, :code:`min-original`, and :code:`tiny-original`
        """
        if self.image is not None:
            return await self._client.get_image(self.slug, type)
        return None

    async def push_changes(self) -> "Recipe":
        return await self._client.update_recipe(self)

    async def get_zip(self) -> ZipFile:
        return await self._client.get_recipe_zip(self.slug)

    async def refresh(self) -> None:
        recipe = await self._client.get_recipe(self.slug)
        for attr in dir(recipe):
            if not attr.startswith("_"):
                setattr(self, attr, getattr(recipe, attr))

    def __repr__(self):
        return f"<Recipe {self.slug!r}>"


@dataclass()
class RecipeTag(JsonModel):
    _client: "MealieClient" = field(repr=False)
    id: int
    name: str
    recipes: t.Optional[t.List[Recipe]] = None

    @property
    def slug(self) -> str:
        return name_to_slug(self.name)

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json({"name", "slug"})

    async def update(self, new_name: str) -> "RecipeTag":
        return await self._client.update_tag(self.slug, new_name)

    async def delete(self):
        await self._client.delete_tag(self.id)


@dataclass()
class RecipeCategory(JsonModel):
    _client: "MealieClient" = field(repr=False)
    id: int
    name: str
    recipes: t.Optional[t.List[Recipe]] = None

    @property
    def slug(self):
        return name_to_slug(self.name)

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json({"name", "slug"})



import io
import typing as t
from dataclasses import dataclass, field
from datetime import timedelta

from mealieapi.auth import Token
from mealieapi.meals import MealPlan, ShoppingList
from mealieapi.mixins import JsonModel
from mealieapi.recipes import Recipe, RecipeCategory

if t.TYPE_CHECKING:
    from mealieapi.client import MealieClient


@dataclass()
class User(JsonModel):
    _client: "MealieClient" = field(repr=False)
    username: str
    full_name: str
    email: str
    admin: bool
    group: str
    id: int
    favorite_recipes: t.Union[t.List[Recipe], None] = None
    tokens: t.Union[t.List[Token], None] = None
    password: t.Union[str, None] = None

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json(
            {
                "username",
                "full_name",
                "email",
                "admin",
                "group",
                "password",
                "id",
                "favorite_recipes",
            }
        )

    async def create(self) -> "User":
        return await self._client.create_user(self)

    async def update(self) -> "User":
        return await self._client.update_user(self.id, self)

    async def delete(self) -> None:
        await self._client.delete_user(self.id)

    async def reset_password(self) -> None:
        await self._client.reset_password(self.id)

    async def update_password(self, new_password: str):
        if self.password:
            await self._client.update_password(self.id, self.password, new_password)
        else:
            raise ValueError("Missing password attribute, required to change password.")

    async def favorites(self) -> t.Optional[t.List[Recipe]]:
        return await self._client.get_favorites(self.id)

    async def add_favorite(self, recipe_slug: str) -> None:
        await self._client.add_favorite(self.id, recipe_slug)

    async def remote_favorite(self, recipe_slug: str) -> None:
        await self._client.remove_favorite(self.id, recipe_slug)

    async def image(self) -> bytes:
        return await self._client.get_user_image(self.id)

    async def update_image(self, image: io.BytesIO) -> bytes:
        return await self._client.update_user_image(self.id, image)


@dataclass()
class Group(JsonModel):
    _client: "MealieClient" = field(repr=False)
    name: str
    id: t.Union[int, None] = None
    categories: t.Union[t.List[RecipeCategory], None] = None
    webhook_urls: t.Union[t.List[str], None] = None
    webhook_time: t.Union[timedelta, None] = None
    users: t.Union[t.List[User], None] = None
    mealplans: t.Union[t.List[MealPlan], None] = None
    shopping_lists: t.Union[t.List[ShoppingList], None] = None
    webhook_enable: t.Union[bool, None] = None

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json({"name", "webhook_urls", "webhook_enable"})

    async def create(self) -> "Group":
        return await self._client.create_group(self)

    async def update(self) -> "Group":
        if self.id is not None:
            return await self._client.update_group(self.id, self)
        else:
            raise ValueError("Missing required parameter id")

    async def delete(self) -> None:
        if self.id is not None:
            await self._client.delete_group(self.id)
        else:
            raise ValueError("Missing required parameter id")


@dataclass()
class UserSignup(JsonModel):
    _client: "MealieClient" = field(repr=False)
    name: str
    admin: bool
    token: t.Union[str, None] = None

    def json(self) -> t.Dict[str, t.Any]:  # type: ignore[override]
        return super().json({"name", "admin"})

    async def delete(self) -> None:
        await self._client.delete_signup_token(self.token)

    async def signup(self, user: User) -> User:
        if self.token:
            return await self._client.signup_with_token(self.token, user)
        else:
            raise ValueError("Tried to signup user without signup token")
